defmodule Datasci.Worker do
  use GenServer
  use Export.Python

  @spec start_link(any()) :: GenServer.on_start()
  def start_link(_) do
    IO.puts("----")
    {:ok, pid} = Python.start(python_path: Path.expand("lib/python"))
    IO.puts("(start_link) starting python -- pid: #{inspect(pid)}")
    value = GenServer.start_link(__MODULE__, pid)
    IO.inspect(value, label: "(start_link) starting worker -- value")
    value                                               
  end

  @spec init(any()) :: {:ok, keyword()}
  def init(pid) do
    IO.puts("(init) pid: #{inspect(pid)}")
    {:ok, [pid: pid]}
  end

  def handle_call({:calc_mean, path, column}, _from, state) do
    pid = Keyword.get(state, :pid)
    {:ok, result} = calc_mean(pid, path, column)
    #Process.sleep(2000)
    {:reply, {:ok, result}, state}
  end

  def terminate(_reason, state) do
    pid = Keyword.get(state, :pid)
    IO.puts("*** stopping python -- pid: #{inspect(pid)}")
    Python.stop(pid)
    :ok
  end

  @doc """
  Calculate the mean of a column in a Pandas data-set.

  ## Examples

      iex> {:ok, result} = DatasciTasks.calc_mean("Data/data_set_01.cvs", 2)
      :world

  """
  @spec calc_mean(pid(), iodata(), String.t()) :: {:ok, []}
  def calc_mean(pid, path, column) do
    path1 = Path.absname(path)
    json_result = Python.call(pid, calc_mean(path1, column), from_file: "datasci_tasks")
    {:ok, result} = Jason.decode(json_result)
    #IO.inspect(result, label: "(calc_mean) result")
    {:ok, result}
  end

end
